#ifndef MSP_GL_SL_TOKENIZER_H_
#define MSP_GL_SL_TOKENIZER_H_

#include <deque>
#include <string>
#include <sigc++/signal.h>

namespace Msp {
namespace GL {
namespace SL {

class Preprocessor;

struct Location
{
	std::string name;
	unsigned line;

	Location(): line(0) { }
	Location(const std::string &n, unsigned l): name(n), line(l) { }
};

class Tokenizer
{
public:
	sigc::signal<void> signal_preprocess;

private:
	std::string::const_iterator iter;
	std::string::const_iterator source_end;
	Location location;
	bool progress_mark;
	bool allow_preprocess;
	bool suppress_line_advance;
	std::string last_token;
	std::deque<std::string> next_tokens;

public:
	Tokenizer();

	void begin(const std::string &, const std::string &);
	const std::string &peek_token(unsigned = 0);
	const std::string &parse_token();
	const std::string &get_last_token() const { return last_token; }
	void expect(const std::string &);
	void set_location(const Location &);
	void clear_progress_mark() { progress_mark = false; }
	bool get_progress_mark() const { return progress_mark; }
	const Location &get_location() const { return location; }
private:
	std::string parse_token_();
	void preprocess();
	std::string parse_identifier();
	std::string parse_number();
	std::string parse_string();
	std::string parse_other();
	void skip_comment_and_whitespace();
};

} // namespace SL
} // namespace GL
} // namespace Msp

#endif
